/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {Button, Form, Header, Icon} from 'semantic-ui-react'
import {RootState} from "../../state/reducers";
import {
  reset as resetLoginData,
  setIsCheckingAlreadyLoggedIn,
  setIsLoggingIn,
  setIsRegisterFormDisplayed,
  setPassword,
  setUsername
} from "../../state/actions/loginActions";
import {getUserSettingsAndDataAsync} from "../../state/actions/settingsSite/userSettingsSiteCrudActions";
import {setUserData} from "../../state/actions/settingsSite/userSettingsSiteActions";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import MaterialInput from '../material/materialInput'

import * as loginLayer from '../../communicationLayer/loginLayer'
import {UserDataFullBase} from "../../types/userData";
import {notifyError, notifyWarning} from "../../helpers/notificationHelper";
import {OnKeyPressEvent} from "../../types/reactEvents";
import {delay} from "../../../debug";
import Spinner from "../helpers/spinner";
import * as cryptoHelper from '../../helpers/cryptoHelper'
import {FrontendSettingsManager} from "../../helpers/frontendSettingsManager";
import {MyPopup} from "../helpers/myPopup";
import {getI18n, getRawI18n} from "../../../i18n/i18nRoot";
import Logger from '../../helpers/logger'
import {
  getGlobalSystemSettingsAsync,
  getTestServerPublicSettingsAsync
} from '../../state/actions/globalSystemSettings/globalSystemSettingsCrudActions'
import {LoginTimeoutHelper} from '../../helpers/loginTimeoutHelper'
import {initial} from '../../state/reducers/settingsSite/settingsSiteReducers'
import {reset_Global} from '../../state/reducers/globalActions'


//const css = require('./styles.styl');

export interface MyProps {
  //readonly caption?: string
}


//form autocomplete (username, password) only works with https?
//see https://stackoverflow.com/questions/21191336/getting-chrome-to-prompt-to-save-password-when-using-ajax-to-login/33113374#33113374
const loginFieldId = 'loginFieldId'
const loginFieldPwId = 'loginFieldPwId'
const loginButtonId = 'loginButtonId'

const mapStateToProps = (rootState: RootState, props: MyProps) => {
  return {
    id: rootState.loginDataState.username,
    pw: rootState.loginDataState.password,
    isLogginIn: rootState.loginDataState.isLoggingIn,
    isCheckingAlreadyLoggedIn: rootState.loginDataState.isCheckingAlreadyLoggedIn,
    beforeLoginErrorMessage: rootState.loginDataState.beforeLoginErrorMessage,
    hasNetworkError: rootState.loginDataState.beforeLoginErrorMessage !== '',
    langId: rootState.i18nState.langId
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
                                                                        setUsername,
                                                                        setPassword,
                                                                        getUserSettingsAndDataAsync,
                                                                        resetLoginData,

                                                                        setUserData,
                                                                        setIsCheckingAlreadyLoggedIn,
                                                                        setIsLoggingIn,
                                                                        setIsRegisterFormDisplayed,
                                                                        getGlobalSystemSettingsAsync,
                                                                        getTestServerPublicSettingsAsync,
                                                                        reset_Global,
                                                                      }, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;


export const formSpinnerOpacity = 0.3

class LoginForm extends React.Component<Props, any> {

  idChanged(e: React.FormEvent<HTMLInputElement>): void {
    this.props.setUsername(e.currentTarget.value)
  }

  pwChanged(e: React.FormEvent<HTMLInputElement>): void {
    this.props.setPassword(e.currentTarget.value)
  }

  componentDidMount(): void {

    // setTimeout(() => {
    //   this.loginClicked()
    // }, 200)
  }

  async loginClicked(e?: any): Promise<void> {

    if (e) e.preventDefault()

    this.props.setIsLoggingIn(true)

    // noinspection TsLint
    let userData: UserDataFullBase = null

    try {

      //also not that we check for the crsf token in setup.ts (axios.interceptors.response)
      userData = await
        delay(
          loginLayer.login({
                             token: this.props.id,
                             password: cryptoHelper.hash(this.props.pw)
                           })
        )

      if (userData === null) {
        notifyWarning(getI18n(this.props.langId, 'Wrong username or password'))
      } else {

        const switchedUsers = LoginTimeoutHelper.isFirstLogin === false && userData.id !== initial.userData.id
          && userData.id !== LoginTimeoutHelper.lastKnownUserId

        //if the user changed reset all data
        if (switchedUsers) {
          this.props.reset_Global()
        }

        this.props.setUserData(userData)

        //clear login form
        this.props.resetLoginData()

        //clear frontend settings if the user changed...
        const frontendSettings = FrontendSettingsManager.getFrontendSettings()
        if (frontendSettings.lastLoggedInUserId !== userData.id) {
          Logger.log("the logged in user changed (id)... clearing the frontend settings")
          FrontendSettingsManager.clear()
        }

        FrontendSettingsManager.setLastLoggedInUserId(userData.id)

        //get the server message after login so we don't need to wait for the next interval

        await this.props.getGlobalSystemSettingsAsync()
        await this.props.getTestServerPublicSettingsAsync()

      }
    } catch (err) {
      notifyError(getI18n(this.props.langId, 'Server error'))
    }
    finally {
      this.props.setIsLoggingIn(false)
    }
  }

  render(): JSX.Element {

    return (
      <div className="frosted-glass fh">
        <div className="frosted-glass-inner absolute-sized-centered">

            <div className="corner-div fh">

            {
              this.props.isCheckingAlreadyLoggedIn &&
              <Spinner opacity={formSpinnerOpacity}/>
            }

            {
              this.props.isLogginIn &&
              <Spinner opacity={formSpinnerOpacity}/>
            }
            {
              (!this.props.isCheckingAlreadyLoggedIn && !this.props.isLogginIn) && this.props.hasNetworkError === false &&
              <form action="login" method="post">
                <div className="ui form frosted-glass-form">
                  <div className="form-header">
                    <Header as='h1'>
                      {getI18n(this.props.langId, 'YAPEX')} - {getI18n(this.props.langId, 'Login')}
                    </Header>
                  </div>

                  <Form.Field>
                    <div>
                      <div>
                        <Icon size="massive" name="user circle"/>
                      </div>
                    </div>
                  </Form.Field>

                  <Form.Field>
                    <label>
                      {
                        getI18n(this.props.langId, 'Token')
                      }
                    </label>
                    <MaterialInput id={loginFieldId} name="username" type="text" tabIndex={1}
                                   autocomplete="name"
                                   placeholder="e.g. abcef"
                                   value={this.props.id}
                                   onChange={(e) => this.idChanged(e)}
                                   ref={(input) => {
                                     if (input) {
                                       const field = document.getElementById(loginFieldId)
                                       const fieldPw = document.getElementById(loginFieldPwId)

                                       if (fieldPw && fieldPw === document.activeElement) {
                                         //don't focus username input if we currently focusing pw input
                                         return
                                       }

                                       if (field) field.focus()
                                     }
                                   }}
                    />
                  </Form.Field>

                  <Form.Field className="relative-positioned">
                    <div className="password-forgotten">
                      <MyPopup
                        trigger={<a>
                          {getI18n(this.props.langId, 'Forgot Password?')}
                        </a>}
                        content={
                          <div>
                    <span>
                      {
                        getI18n(this.props.langId, 'Talk to the tutor, he/she will reset your password.')
                      }
                    </span>
                          </div>
                        }
                        on='click'
                      />
                    </div>
                    <label>
                      {
                        getI18n(this.props.langId, 'Password')
                      }
                    </label>
                    <MaterialInput id={loginFieldPwId} name="password" type="password" tabIndex={2}
                                   autocomplete="password"
                                   placeholder={`${getI18n(this.props.langId, 'Password')}...`}
                                   value={this.props.pw}
                                   onChange={e => this.pwChanged(e)}

                                   onKeyPress={(e: OnKeyPressEvent) => {
                                     if (e.key === 'Enter') {
                                       const btn = document.getElementById(loginButtonId)
                                       if (btn) btn.click()
                                     }
                                   }}
                    />


                  </Form.Field>

                  <div>
                    <Button color="blue"
                            id={loginButtonId}
                            type="submit"
                            onClick={e => this.loginClicked(e)}
                            content={getI18n(this.props.langId, 'Sign In')}
                    >
                    </Button>
                    <Button
                      onClick={() => {
                        this.props.setIsRegisterFormDisplayed(true)
                      }}
                    >

                    <span>
                      {
                        getI18n(this.props.langId, 'Register')
                      }
                    </span>
                      <Icon name="chevron right"/>
                    </Button>
                  </div>

                </div>
              </form>
            }
            {
              this.props.hasNetworkError &&
                <div className="fh full-w flexed">
                  <p className="white-colored v-centered-i" style={{fontSize: '16px', textAlign: 'center'}}>
                    {
                      this.props.beforeLoginErrorMessage
                    }
                  </p>
                </div>
            }
          </div>


        </div>

        <div className="absolute-sized-centered-below-login cookie-info-wrapper">
          <div>
            <span dangerouslySetInnerHTML={getRawI18n(this.props.langId,
                                                      'We use cookies to store information across sessions.<br /> By signing in you agree to'
            )}></span>
            <a href='/copyright.html' target="_blank" className="terms-of-use-link">{
              ' ' + getI18n(this.props.langId, 'our use of cookies.')
            }</a>
          </div>
        </div>

      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(LoginForm)

