import {ActionBase, ActionType} from "./dashboardSiteActionTypes";
import {
  ClientServerAppSettings,
  ClientServerConstants,
  HardwareUsageStatistic, HardwareUsageStatisticFrontendExclusive,
  LogicalUsageStatistic, TestServerPrivateSettings
} from "../../../types/dashboard";

import {
  AllActions as AllGetClientServerSettingsActions,
  reducer as getClientServerSystemSettingsReducer
} from "./crud/getClientServerSystemSettingsReducer"
import {notExhaustive} from "../_notExhausiveHelper";

import {AllActions as getHardwareUsageActions, reducer as getHardwareUsageReducer} from './crud/getHardwareUsageReducer'

import {AllActions as getLogicalUsageActions, reducer as getLogicalUsageReducer} from './crud/getLogicalUsageReducer'

import {
  AllActions as getTestServerSettingsActions,
  reducer as getTestServerSettingsReducer
} from './crud/getTestServerSettingsReducer'
import {SystemSettingsFromBackend} from '../../../types/systemSettings'

import {
  AllActions as AllGetClientServerConstantsActions,
  reducer as getClientServerConstantsReducer
} from './crud/getClientServerConstantsReducer'

import {
  AllActions as AllGetClientServerAppSettingsActions,
  reducer as getClientServerAppSettingsReducer
} from './crud/getClientServerAppSettingsReducer'
import {ActionTypeGlobal, RESET_GlobalAction} from '../globalActions'


export type State = {
  readonly isLoading: boolean

  readonly hardwareUsageStatistic: ReadonlyArray<HardwareUsageStatisticFrontendExclusive>

  /**
   * true: refresh the hardware usage view in intervals,
   * @see dashboardRefreshIntervalInS
   * false: not
   */
  readonly isHardwareUsageStatisticRefreshing: boolean

  readonly logicalUsageStatistic: LogicalUsageStatistic | null

  readonly clientServerSystemSettings: SystemSettingsFromBackend | null

  readonly clientServerConstants: ClientServerConstants | null

  readonly clientServerAppSettings: ClientServerAppSettings | null

  readonly testServerSettings: TestServerPrivateSettings | null

}


export const initial: State = {
  isLoading: false,

  hardwareUsageStatistic: [],
  isHardwareUsageStatisticRefreshing: false,

  logicalUsageStatistic: null,
  clientServerSystemSettings: null,
  clientServerConstants: null,
  clientServerAppSettings: null,

  testServerSettings: null
}


export interface SET_isLoadingAction extends ActionBase {
  readonly type: ActionType.SET_isLoading
  readonly isLoading: boolean
}


export interface SET_isHardwareUsageStatisticRefreshingAction extends ActionBase {
  readonly type: ActionType.SET_isHardwareUsageStatisticRefreshing
  readonly isRefreshing: boolean
}


export interface ResetAction extends ActionBase {
  readonly type: ActionType.RESET
}

export type AllActions =
  ResetAction

  | SET_isLoadingAction
  | SET_isHardwareUsageStatisticRefreshingAction

  | AllGetClientServerConstantsActions
  | AllGetClientServerAppSettingsActions
  | AllGetClientServerSettingsActions
  | getHardwareUsageActions
  | getLogicalUsageActions
  | getTestServerSettingsActions

  | RESET_GlobalAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.GET_clientServerConstants_PENDING:
    case ActionType.GET_clientServerConstants_FULFILLED:
    case ActionType.GET_clientServerConstants_REJECTED:
      return getClientServerConstantsReducer(state, action)

    case ActionType.GET_clientServerAppSettings_PENDING:
    case ActionType.GET_clientServerAppSettings_FULFILLED:
    case ActionType.GET_clientServerAppSettings_REJECTED:
      return getClientServerAppSettingsReducer(state, action)

    case ActionType.GET_clientServerSettings_PENDING:
    case ActionType.GET_clientServerSettings_FULFILLED:
    case ActionType.GET_clientServerSettings_REJECTED:
      return getClientServerSystemSettingsReducer(state, action)

    case ActionType.GET_testServerSettings_PENDING:
    case ActionType.GET_testServerSettings_FULFILLED:
    case ActionType.GET_testServerSettings_REJECTED:
      return getTestServerSettingsReducer(state, action)

    case ActionType.GET_logicalUsage_PENDING:
    case ActionType.GET_logicalUsage_FULFILLED:
    case ActionType.GET_logicalUsage_REJECTED:
      return getLogicalUsageReducer(state, action)

    case ActionType.GET_hardwareUsage_PENDING:
    case ActionType.GET_hardwareUsage_FULFILLED:
    case ActionType.GET_hardwareUsage_REJECTED:
      return getHardwareUsageReducer(state, action)


    case ActionType.SET_isLoading:
      return {
        ...state,
        isLoading: action.isLoading
      }

    case ActionType.SET_isHardwareUsageStatisticRefreshing:
      return {
        ...state,
        isHardwareUsageStatisticRefreshing: action.isRefreshing
      }

    case ActionType.RESET:
    case ActionTypeGlobal.RESET_Global:
      return {
        ...initial
      }

    default:
      notExhaustive(action)
      return state
  }
}

